#include "XineStream.h"

#include "Application.h"

#include <xine.h>
#include <xine/xineutils.h>

#if 0
/*
statut:
idle:		nothing to play / playback finished
playing:	playing media
paused:		playing media but paused

*/
class XineStatus {
public:
    xine_t *xine;
    xine_stream_t *stream;
    xine_video_port_t *vo_port;
    xine_audio_port_t *ao_port;
    xine_event_queue_t *event_queue;
    
    XineStatus() {
    }
    
};

static void xine_call_back(void * user_data,const xine_event_t * evt)
{
    XineStream * x = (XineStream*)user_data;
    if (evt->type == XINE_EVENT_UI_PLAYBACK_FINISHED) {
	// FIXME :  emit thread notification for
	jsrlog("xine_ui : received finished callback, status is %d\n",x->getPlaying());
	x->playbackFinished.emitThreadNotification();
    }
}

XineStream::XineStream() : JSReflection::Object(Application::getJsContext())
{
    playbackFinished.setTarget(this);
    xine = new XineStatus();
    
    xine->xine = xine_new();
    //xine_engine_set_param(xine->xine,XINE_ENGINE_PARAM_VERBOSITY,XINE_VERBOSITY_DEBUG );
    xine_config_load(xine->xine, "./config-xine");
    xine_init(xine->xine);
    
    xine->vo_port = 0;
    xine->ao_port = xine_open_audio_driver(xine->xine,"auto",NULL);
    xine->stream = xine_stream_new(xine->xine, xine->ao_port, xine->vo_port);
    
    xine->event_queue = xine_event_new_queue(xine->stream);
    xine_event_create_listener_thread(xine->event_queue,&xine_call_back,(void*)this);
}

XineStream::~XineStream()
{
    // FIXME : todo !
}

void XineStream::notified(Notifier * n)
{
    jsrlog("XineStream notified !\n");
    if (n == &playbackFinished) {
	// Check that no new stream was started between event and notification !
	if ((onPlaybackFinished.length() > 0) && !getPlaying()) {
	    evaluate(onPlaybackFinished.c_str());
	}
	return;
    }
}

bool XineStream::open(const std::string & path)
{
    jsrlog("opening %s\n",path.c_str());
    if (xine_open(xine->stream, path.c_str())) {
	xine_set_param(xine->stream,XINE_PARAM_IGNORE_VIDEO,1);
	return true;
    }
    return false;
}

bool XineStream::play()
{
    return xine_play(xine->stream, 0, 0);
}

void XineStream::stop()
{
    xine_stop(xine->stream);    
}

void XineStream::pause()
{
    xine_set_param(xine->stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
}

void XineStream::resume()
{
    xine_set_param(xine->stream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
}

int XineStream::getDuration()
{
    int pos_stream,pos_time,length;
    if (xine_get_pos_length(xine->stream,&pos_stream,&pos_time,&length)) {
	return length;
    } else {
	return 0;
    }
}

int XineStream::getPosition()
{
    int pos_stream,pos_time,length;
    if (xine_get_pos_length(xine->stream,&pos_stream,&pos_time,&length)) {
	return pos_time;
    } else {
	return 0;
    }
}

StreamStatus XineStream::getStreamStatus()
{
    int i;
    i = xine_get_status(xine->stream);
    if (i != XINE_STATUS_PLAY) {
        if (xine_get_param(xine->stream, XINE_PARAM_SPEED) != XINE_SPEED_PAUSE) {
            return StreamPlaying;
        }
        return StreamPaused;
    }
    return StreamIdle;
}

std::string XineStream::getStrStreamStatus()
{
    // FIXME : clear pending notification !
    switch(getIStatus()) {
      case Status_Idle:
        return "idle";
      case Status_Playing:
        return "playing";
      case Status_Paused:
        return "paused";
      default:
        return "idle;"
    }
}

std::string XineStream::getOnStatusChanged() const
{
    return onStatusChanged;
}

void XineStream::setOnStatusChanged(const std::string & n)
{
    onStatusChanged = n;
}
    
static XineStream * newStream()
{
    return new XineStream();
}

JS_EXPORT_CLASS(XineStream,JSReflection::Object);
JS_RO_PROPERTY(XineStream,duration,getDuration,int,"%i",0);
JS_RO_PROPERTY(XineStream,position,getPosition,int,"%i",0);
JS_RO_PROPERTY(XineStream,status,getStrStreamStatus,JSReflection::JS_String,"%s",0);
JS_RW_PROPERTY(XineStream,onStatusChanged,getOnStatusChanged,setOnStatusChanged,JSReflection::JS_String,"%s",0);
JS_NOCONTEXT_GLOBAL_FUNCTION_0(newStream,XineStream*,"%x",XineStream::ClassInfo);
JS_NOCONTEXT_OBJECT_FUNCTION_1(XineStream,open,bool,"%b",0,JSReflection::JS_String,"%s",0);
JS_NOCONTEXT_OBJECT_FUNCTION_0(XineStream,play,bool,"%b",0);
JS_NOCONTEXT_OBJECT_VOID_FUNCTION_0(XineStream,stop);
JS_NOCONTEXT_OBJECT_VOID_FUNCTION_0(XineStream,pause);
JS_NOCONTEXT_OBJECT_VOID_FUNCTION_0(XineStream,resume);
#endif
