#ifndef APPLICATION_H
#define APPLICATION_H 1

#include <string>
#include <set>
#include <vector>

#include "jsreflect.h"

#include "Timer.h"
#include "Widget.h"
#include "Producer.h"

class InputMap;

class ScreenUpdateCallBack {
    bool registered;
public:
    ScreenUpdateCallBack();
    virtual ~ScreenUpdateCallBack();
    
    void listen();
    void unlisten();
    
    virtual void screenUpdateSoon() = 0;
};

/**
 * Schedules and performs screen updates of the current Application
 */
class Updater:public Timer {
    friend class ScreenUpdateCallBack;
    virtual void tick();
    long long updateDelay;
    std::set<ScreenUpdateCallBack*> updateCallBacks;
    /// During callback iteration, keep list of callback to call
    std::vector<ScreenUpdateCallBack*> * currentIteration;
    
    void insertCallBack(ScreenUpdateCallBack * b);
    void removeCallBack(ScreenUpdateCallBack * b);
public:
    Updater();
    /// Set the minimal delay between scheduled updates.
    void setDefaultDelay(int ms);
    
    /// If immediate is true, the screen will be updated without long delay (< 1ms)
    void ask(bool immediate = false);
    
};

/**
 * Main application
 */
class Application {
    friend class Timer;
    friend class Updater;
    friend class Selectable;
    
    long long lastNow;
    long long dayInc;
    
    int inputDisabled;
    
    std::set < Timer* > programmedEvents;
    std::set < Selectable* > selectables;
    Widget * root;
    
    
    SDL_Surface * screen;
    Surface * screenSurface;
    InputMap * inputMap;
    
    void redraw();
    void changeVideoMode(int, int);
public:
    
    Application();
    
    long long now(bool update=false);
    
    void initialise(int scw,int sch);
    
    void setRoot(Widget * w);
    Widget * getRoot() const;
    
    void inputDisable(int n);
    
    void run();
    
    Updater updater;
    
    void show();
    void hide();
    
    bool simulateFullscreen;
    bool isFullscreen;
    void setFullscreen(bool on);
    
    static Application * current;
    static JSReflection::Context * getJsContext();
    static char * slang_tmp_string;
    static int slang_tmp_int;
    void dorun();
    
    void inputEvent(const std::string & key);
    
    static void reportFileChanged(const std::string &);
    
    static bool loadXml(const char * xmlFileName, Widget * w = 0);
    static Widget * loadXmlWidget(const char * xmlFileName, Widget * w = 0);
    
    JSReflection::Context * jsContext;
    InputMap * getInputMap() const;
};

#endif

