#include <SDL.h>
#include <SDL_image.h>

#include "Graphs.h"
#include "Application.h"

#include <string>

bool Font::initialised = false;

Font::Font(): JSReflection::Object(Application::getJsContext()) {
    TTFFont = 0;
    if (!initialised) {
	TTF_Init();
	initialised = true;
    }
}

Font::~Font() {
    if (TTFFont) {
	TTF_CloseFont(TTFFont);
    }
}

bool Font::Load(const char * filename,int size)
{
    if (!initialised) {
	TTF_Init();
	initialised = true;
    }

    if (TTFFont) {
	TTF_CloseFont(TTFFont);
	TTFFont = 0;
    }
    
    TTFFont = TTF_OpenFont(filename,size);
    if (!TTFFont) {
	fprintf(stderr,"TTF_OpenFont: %s\n", TTF_GetError());
    }
    return (TTFFont != 0);
}
    
void Font::SetStyle(int style)
{
    if (!TTFFont) return;
	
    TTF_SetFontStyle(TTFFont,style);
}

int Font::GetStyle() const
{
    if (!TTFFont) return 0;
    
    return TTF_GetFontStyle(TTFFont);
}

bool Font::IsBold() const
{
    return GetStyle() & TTF_STYLE_BOLD;
}

void Font::SetBold(bool b)
{
    SetStyle((GetStyle() & ~TTF_STYLE_BOLD) | (b?TTF_STYLE_BOLD:0));
}

bool Font::IsItalic() const
{
    return GetStyle() & TTF_STYLE_ITALIC;
}

void Font::SetItalic(bool b)
{
    SetStyle((GetStyle() & ~TTF_STYLE_ITALIC) | (b?TTF_STYLE_ITALIC:0));
}

bool Font::IsUnderline() const
{
    return GetStyle() & TTF_STYLE_UNDERLINE;
}

void Font::SetUnderline(bool b)
{
    SetStyle((GetStyle() & ~TTF_STYLE_UNDERLINE) | (b?TTF_STYLE_UNDERLINE:0));
}

void Font::sizeText(const std::string & text,int & w,int & h) const
{
    if ((!TTFFont) || (!strlen(text.c_str()))) {
	w = 0;
	h = 0;
	return;
    }
    TTF_SizeUTF8(TTFFont,text.c_str(), &w, &h);
}


TTF_Font * Font::GetFont() const
{
    return TTFFont;
}

Font * Font::LoadFont(const char * filename,int size)
{
    Font * f = new Font();
    if (f->Load(filename,size)) {
	return f;
    }
    delete(f);
    return 0;
}

static Font * loadFont(const char* filename,int size) {
    Font * ret =  Font::LoadFont(filename,size);
    //printf("loadFont returning %#x\n",ret);
    return ret;
}

JS_EXPORT_CLASS(Font,JSReflection::Object)
JS_NOCONTEXT_GLOBAL_FUNCTION_2(loadFont,Font*,"%x",Font::ClassInfo,
				JSReflection::JS_String,"%s",0,
				int,"%i",0)

