#include <SDL.h>
#include <SDL_image.h>

#include "Graphs.h"
#include "Application.h"
#include "Widget.h"
#include "InputMap.h"
#include <string>


InputEventHandler::InputEventHandler(Widget * f) : JSReflection::Object(Application::getJsContext()), name()
{
    father = f;
    allowUnvisibleFlag = false;
    f->addInputEventHandler(this);
}

InputEventHandler::~InputEventHandler()
{
    if (father) {
	father->removeInputEventHandler(this);
    }
}

Widget * InputEventHandler::getFather() const
{
    return father;
}

void InputEventHandler::setName(const std::string & vname) 
{
    name = vname;
}

std::string InputEventHandler::getName() const
{
    return name;
}

void InputEventHandler::setAction(const std::string & vaction) 
{
    action = vaction;
}

std::string InputEventHandler::getAction() const
{
    return action;
}

void InputEventHandler::setAllowUnvisible(bool b)
{
    allowUnvisibleFlag = b;
}

bool InputEventHandler::allowUnvisible() const
{
    return allowUnvisibleFlag;
}

bool InputEventHandler::match(const std::string & event, bool visibility)
{
    if (!visibility && !allowUnvisibleFlag) {
	return false;
    }
    return Application::current->getInputMap()->inputMatchKey(name,event);
    //return event == name;
}

bool InputEventHandler::call(const std::string & event)
{
    GetContext()->ExecuteScript(action.c_str(),father);
    // Fixme : need a callback !
    return true;
}


JS_EXPORT_CLASS(InputEventHandler,JSReflection::Object)
JS_RO_PROPERTY(InputEventHandler,father,getFather,Widget*,"%x",Widget::ClassInfo);
JS_RW_PROPERTY(InputEventHandler,allowunvisibleevent,allowUnvisible,setAllowUnvisible,bool,"%b",0);
JS_RW_PROPERTY(InputEventHandler,name,getName,setName,JSReflection::JS_String,"%s",0);
JS_RW_PROPERTY(InputEventHandler,action,getAction,setAction,JSReflection::JS_String,"%s",0);
