
#include "jsreflect.h"
#include "XmlParser.h"
#include "Application.h"
#include "InputMap.h"

InputMap::InputMap()
{
}

InputMap::~InputMap()
{
}

bool InputMap::singleMatch(const std::string & input,const std::string & key)
{
    if (mapping.find(input) != mapping.end()) {
	for(typeof(mapping[input].begin()) it  = mapping[input].begin(); it != mapping[input].end(); ++it) {
	    if ((*it) == key) {
		return true;
	    }
	}
    }
    return false;
}

bool InputMap::inputMatchKey(const std::string & input,const std::string & key)
{
    if (singleMatch(input,key)) {
	return true;
    }
    
    return false;
}

class InputItem : public XmlParsableItem {
protected:
    InputMap * addTo;
    std::string input_name;
    bool hasname;
    std::list<std::string> values;
public:
    InputItem(InputMap * to) : XmlParsableItem() {
	addTo = to;
	hasname = false;
    }
    
    virtual void addAttribute(const std::string & name,const std::string & value) {
	if (name == "name") {
	    input_name = value;
	    hasname = true;
	} else if (name.substr(0,3) == "key") {
	    values.push_back("KEY:" + value);
	} else if (name.substr(0,4) == "lirc") {
	    values.push_back("LIRC:" + value);
	}
    }
    
    virtual void done() {
	if (hasname) {
	    for(typeof(values.begin()) it = values.begin(); it != values.end(); ++it) {
		addTo->add(input_name,*it);
	    }
	}
    }
};

class MapDocument: public XmlParsableItem {
protected:
    InputMap * addTo;
public:
    MapDocument(InputMap * to) : XmlParsableItem() {
	addTo = to;
    }
    
    XmlParsableItem * newChild(const std::string & name) {
	if (name == "input") {
	    return new InputItem(addTo);
	}
	return XmlParsableItem::newChild(name);
    }
};

class MapRoot: public XmlParsableItem {
protected:
    InputMap * addTo;
public:
    MapRoot(InputMap * to) : XmlParsableItem() {
	addTo = to;
    }
    
    XmlParsableItem * newChild(const std::string & name) {
	if (name == "map") {
	    return new MapDocument(addTo);
	}
	return XmlParsableItem::newChild(name);
    }
};

void InputMap::add(const std::string & input,const std::string & key)
{
    jsrlog("input map add key %s for function %s\n",key.c_str(),input.c_str());
    mapping[input].push_back(key);
}

void InputMap::load(const std::string & path)
{
    XmlParser parser;
    MapRoot * root = new MapRoot(this);
    parser.pushItem(root);
    parser.parseDocument(path);
}

static int loadInputMap(const std::string & path)
{
    Application::current->getInputMap()->load(path);
    return 0;
}

JS_NOCONTEXT_GLOBAL_FUNCTION_1(loadInputMap,int,"%i",0,JSReflection::JS_String,"%s",0)

