#include <SDL.h>
#include <SDL_image.h>

#include "jsreflect.h"
#include "XmlParser.h"
#include "Graphs.h"
#include "Application.h"
#include "Widget.h"
#include "GridWidget.h"
#include "ImageWidget.h"
#include "Layout.h"
#include "Style.h"
#include "JWidgetClass.h"
#include <string>

std::map<std::string,WidgetClass*> WidgetClass::classes;

bool WidgetClass::initialised = false;

WidgetClass::WidgetClass(const std::string & name)
{
    classes[name] = this;
}

WidgetClass::~WidgetClass()
{
}

class WidgetClass_Widget : public WidgetClass {
public:
    virtual ~WidgetClass_Widget() {}
    WidgetClass_Widget(const std::string & name) : WidgetClass(name) {}
    
    virtual Widget * createInstance(Widget * f) 
    {
	Widget * ret = new Widget();
	ret->setFather(f);
	return ret;
    }
};

class WidgetClass_Image : public WidgetClass {
public:
    WidgetClass_Image(const std::string & name) : WidgetClass(name) {}
    virtual ~WidgetClass_Image() {}
    virtual Widget * createInstance(Widget * f) { 
    	Widget * ret = new ImageWidget();
	ret->setFather(f);
	return ret;
    }
};

class WidgetClass_Grid : public WidgetClass {
public:
    WidgetClass_Grid(const std::string & name) : WidgetClass(name) {}
    virtual ~WidgetClass_Grid() {}
    virtual Widget * createInstance(Widget * f) { 
    	Widget * ret = new GridWidget();
	ret->setFather(f);
	return ret;
    }
};

void WidgetClass::initialise() 
{
    if (!initialised) {
	initialised = true;
	new WidgetClass_Widget("widget");
	new WidgetClass_Image("image");
	new WidgetClass_Grid("grid");
    }
}

static Widget * newWidgetInstance(const std::string & str,Widget * father)
{
    typeof(WidgetClass::classes.find(str)) it = WidgetClass::classes.find(str);
    if (it == WidgetClass::classes.end()) {
	return 0;
    }
    return it->second->createInstance(father);
}

class JWidgetClass : public WidgetClass, public JSReflection::Object 
{
    JS_EXPORTED_CLASS
    
public:
    JWidgetClass(const std::string & name) : WidgetClass(name), JSReflection::Object(Application::getJsContext())
    { }
    
    virtual ~JWidgetClass()
    { }
    
    virtual Widget * createInstance(Widget * f) {
	Widget * obj;
	jsrlog("createInstance\n");
	if (call("create",&obj,"%x",Widget::ClassInfo,"%x",&f,Widget::ClassInfo)) {
	    jsrlog("call succeded\n");
	    return obj;
	} else {
	    return 0;
	}
    }
};

static JWidgetClass * newWidgetClass(const std::string & name)
{
    return new JWidgetClass(name);
}

JS_EXPORT_CLASS(JWidgetClass,JSReflection::Object);
JS_NOCONTEXT_GLOBAL_FUNCTION_1(newWidgetClass,JWidgetClass*,"%x",JWidgetClass::ClassInfo,JSReflection::JS_String,"%s",0);
JS_NOCONTEXT_GLOBAL_FUNCTION_2(newWidgetInstance,Widget*,"%x",Widget::ClassInfo,JSReflection::JS_String,"%s",0,Widget*,"%x",Widget::ClassInfo);

