#include <SDL.h>
#include <SDL_image.h>
#include <SDL_gfxPrimitives.h>

#include <string>
using namespace std;

#include "Graphs.h"
#include "Widget.h"


Painter::Painter(Surface * surface,const ScreenArea * dc)
{
    OwnSurf = false;
    Surf = surface->Surf;
    ownArea = false;
    HasArea = (dc != 0);
    Area = dc;
    XShift = 0;
    YShift = 0;
    XSurfShift = 0;
    YSurfShift = 0;
}

Painter::Painter(Widget * surface,Surface * dest,const ScreenArea * dc)
{
    surface->getAbsolutePos(XShift,YShift);
    
    ScreenArea tmp(XShift,YShift,surface->w,surface->h);
    
    Area = dc->Intersect(&tmp);
    
    OwnSurf = false;
    Surf = dest->Surf;
    HasArea = true;
    ownArea = true;
     
    XShift = -XShift;
    YShift = -YShift;
    XSurfShift = 0;
    YSurfShift = 0;
}

Painter::~Painter()
{
    if (OwnSurf && Surf) {
	SDL_FreeSurface(Surf);
    }
    if (ownArea) {
	delete(Area);
    }
}

void Painter::SetRelative(int x,int y) 
{
    XShift = x;
    YShift = y;
}

void Painter::SetRelativeX(int x){
    XShift = x;
}

int Painter::GetRelativeX() const {
    return XShift;
}

void Painter::SetRelativeY(int y){
    YShift = y;
}

int Painter::GetRelativeY() const {
    return YShift;
}

void Painter::SetSurfaceRelative(int x,int y) 
{
    XSurfShift = x;
    YSurfShift = y;
}

void Painter::SetSurfaceRelativeX(int x){
    XSurfShift = x;
}

int Painter::GetSurfaceRelativeX() const {
    return XSurfShift;
}

void Painter::SetSurfaceRelativeY(int y){
    YSurfShift = y;
}

int Painter::GetSurfaceRelativeY() const {
    return YSurfShift;
}

void Painter::shift(int x,int y)
{
    XShift += x;
    YShift += y;
}

bool Painter::Intersect( const Rectangle & relativeTarget,
		const Rectangle & target, 
		SDL_Rect & realTarget, SDL_Rect & resulting)
{
    int x0 = relativeTarget.X;
    int y0 = relativeTarget.Y;
    int w = relativeTarget.W;
    int h = relativeTarget.H;
    
   
    if (x0 >= target.X + target.W) return false;
    if (y0 >= target.Y + target.H) return false;
    if (x0 + w <= target.X) return false;
    if (y0 + h <= target.Y) return false;
    
    if (x0 + w > target.X + target.W) {
	w = target.X + target.W - x0;
    }
    
    if (y0 + h > target.Y + target.H) {
	h = target.Y + target.H - y0;
    }
    
    if (x0 < target.X) {
	resulting.x = (target.X - x0);
	w -= (target.X - x0);
	x0 = target.X;
    } else {
	resulting.x = 0;
    }
    
    if (y0 < target.Y) {
	resulting.y = (target.Y - y0);
	h -= (target.Y - y0);
	y0 = target.Y;
    } else {
	resulting.y = 0;
    }
    
    resulting.w = w;
    resulting.h = h;
    realTarget.x = x0 - XSurfShift;
    realTarget.y = y0 - YSurfShift;
    realTarget.w = w;
    realTarget.h = h;
    return true;
}

void Painter::FillRectangle(int x0,int y0,int w,int h,const Color & c)
{
    if (!Surf) { return; }
    
    x0 -= XShift;
    y0 -= YShift;
    Rectangle localTarget(x0,y0,w,h);
    
    if (HasArea) {
	//Uint32 fillValue = c.map(Surf);
	for ( int i = 0;i < Area->RectangleCount; i++ )
	{
	    SDL_Rect target,localpos;
	    if (Intersect(localTarget,Area->Rectangles[i],target,localpos)) {
		if (c.a() == SDL_ALPHA_OPAQUE) {
		    Color nc = c;
		    nc.setA(SDL_ALPHA_OPAQUE);
		    SDL_FillRect(Surf, &target, c.map(Surf));
		} else {
		    boxColor(Surf, target.x,target.y,target.x + target.w - 1,target.y + target.h - 1, c.as_uint());
		}
	    }
	}
    }
}

void Painter::DrawText(const char * text,int x,int y,Font * f,const Color & c,int halign,int valign)
{
    if (!(f && Surf && f->GetFont())) return;

    SDL_Surface * surf = TTF_RenderUTF8_Blended(f->GetFont(),text,c.as_SDL_Color());
    if (!surf) return;

    int vx,vy;

    if (halign < 0) {
	// Left assignment
	vx = x;
    } else if (halign > 0) {
	vx = x - surf->w + 1;
    } else {
	vx = x - surf->w / 2;
    }

    if (valign < 0) {
	// Left assignment
	vy = y;
    } else if (valign > 0) {
	vy = y - surf->h + 1;
    } else {
	vy = y - surf->h / 2;
    }
	

    SDLBlit(surf,vx,vy,-1,-1,0,0);
    SDL_FreeSurface(surf);
}

void Painter::Blit(Surface * f,int x0,int y0,int w,int h, int surfx0,int surfy0)
{
    SDLBlit(f->Surf,x0,y0,w,h,surfx0,surfy0);
}

void Painter::SDLBlit(SDL_Surface * f,int x0,int y0,int w,int h, int surfx0,int surfy0)
{
    if (!(f && Surf)) { return; }
    
    x0 -= XShift;
    y0 -= YShift;
    
    if (w == -1) { w = f->w; }
    if (w > f->w - surfx0) {
	if (surfx0 >= f->w) {
	    return;
	}
	w = f->w - surfx0;
    }
    
    if (h == -1) { h = f->h; }
    if (h > f->h - surfy0) {
	if (surfy0 >= f->h) {
	    return;
	}
	h = f->h - surfy0;
    }
    
    Rectangle localTarget(x0,y0,w,h);
    if (HasArea) {

	for ( int i = 0;i < Area->RectangleCount; i++ )
	{
	    SDL_Rect target,localpos;
	    if (Intersect(localTarget,Area->Rectangles[i],target,localpos)) {
		localpos.x += surfx0;
		localpos.y += surfy0;
		SDL_BlitSurface( f, &localpos, Surf, &target);
	    }
	}
    }
}

const ScreenArea * Painter::getArea() const
{
    return Area;
}
