#include <SDL.h>
#include <SDL_thread.h>
#include "Application.h"
#include "Notifications.h"
#include "Processes.h"

#include <map>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>

static std::map<int, ChildProcess *> pids;

ChildProcess::~ChildProcess()
{
    InternalTask::lock();
    if (pid != -1) {
	pids.erase(pid);
    }
    InternalTask::unlock();
}

ChildProcess::ChildProcess()
{
    pid = -1;
    ret = -1;
    dettached = false;
}

bool ChildProcess::suicide()
{
    return dettached;
}

void ChildProcess::detach()
{
    InternalTask::lock();
    if (isRunning()) {
	dettached = true;
    } else {
	delete(this);
    }
    InternalTask::unlock();
}

int ChildProcess::getReturnCode() const
{
    int r;
    InternalTask::lock();
    r = ret;
    InternalTask::unlock();
    return r;
}

int ChildProcess::fork()
{
    int f;
    InternalTask::lock();
    f = ::fork();
    if (f == -1)  {
	perror("fork");
    } else {
	if (f != 0) {
	    pids[f] = this;
	    pid = f;
	    ret = -1;
	}
    }
    InternalTask::unlock();
    return f;
}

static void sigchldhldr(int)
{
    ChildProcess::sigChld();
}

void ChildProcess::sigChld()
{
    pid_t pid;
    int status,ret;
    jsrlog("sigChld\n");
    
    InternalTask::lock();
    while ((pid = waitpid(-1,&status,WNOHANG))> 0) {
	if (pids.find(pid) != pids.end()) {
	    ChildProcess * cp = pids[pid];
	    cp->pid = -1;
	    if (WIFEXITED(status)) {
		ret = WEXITSTATUS(status);
	    } else {
		ret = -1;
	    }
	    cp->ret = ret; 
	    pids.erase(pid);
	    cp->autoSignal();
	}
    }
    InternalTask::unlock();
}

void ChildProcess::installSigHandler()
{
    struct sigaction act;
    act.sa_handler = &sigchldhldr;
    sigemptyset(&act.sa_mask);
    act.sa_flags = 0;
    sigaction(SIGCHLD,&act,0);
    maskSignal();
}

void ChildProcess::maskSignal()
{
    sigset_t set;
    sigemptyset(&set);
    sigaddset(&set, SIGCHLD);
    sigprocmask(SIG_BLOCK,&set,0);
}

void ChildProcess::unmaskSignal()
{
    sigset_t set;
    sigemptyset(&set);
    sigaddset(&set, SIGCHLD);
    sigprocmask(SIG_UNBLOCK,&set,0);
}

