#include <SDL.h>
#include <SDL_image.h>


#include <stdexcept>

using namespace std;

#include "Graphs.h"
#include "Timer.h"
#include "Application.h"

#include <sys/time.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

Timer::Timer()
{
    hasTimeout = 0;
    timeoutAt = 0;
}

Timer::~Timer()
{
    jsrlog("Deleting timer.\n");
    if (hasTimeout) 
    {
	hasTimeout = false;
	Application::current->programmedEvents.erase(this);
    }
    jsrlog("done.\n");
}

void Timer::setNextTick(long long when)
{
    timeoutAt = when;
    
    if (!hasTimeout) 
    {
	hasTimeout = true;
	
	Application::current->programmedEvents.insert(this);
    }
}

bool Timer::programmed() const
{
    return hasTimeout;
}

long long Timer::nextTick() const
{
    if (!hasTimeout) {
	throw logic_error("Timer::nextTick called without programmed tick.");
    }
    return timeoutAt;
}

long long Timer::currentTick() const
{
    if (hasTimeout) {
	throw logic_error("Timer::currentClick called outside of tick().");
    }
    return timeoutAt;
}

/**
 * The default implementation does nothing.
 */
void Timer::tick()
{
}


