#include <SDL.h>
#include <SDL_image.h>


#include <stdexcept>

using namespace std;

#include "Graphs.h"
#include "Widget.h"
#include "Application.h"

#include <sys/time.h>
#include <errno.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

Updater::Updater():Timer(),updateCallBacks()
{
    /// 60ms is less than TV image frequency.
    updateDelay = 60;
    currentIteration = 0;
}

/**
 * ask for a screen update.
 * The screen update will occure on the next mainloop (if immediate),
 * or within the next updateDelay.
 */
void Updater::ask(bool immediate)
{
    jsrlog("Updater::ask %i\n",immediate);
    
    if (immediate) {
	setNextTick(Application::current->now());
    } else {
	long long newprog = Application::current->now() + updateDelay;
	if (programmed()) {
	    if (newprog < nextTick()) {
		setNextTick(newprog);
	    }
	} else {
	    setNextTick(newprog);
	}
    }
}

void Updater::insertCallBack(ScreenUpdateCallBack * b)
{
    updateCallBacks.insert(b);
}

void Updater::removeCallBack(ScreenUpdateCallBack * b)
{
    updateCallBacks.erase(b);
    
    if (currentIteration) {
	for(vector<ScreenUpdateCallBack*>::iterator it = currentIteration->begin();it != currentIteration->end(); ++it) {
	    if ((*it) == b) {
		(*it) = NULL;
	    }
	}
    }
}

void Updater::tick()
{
    vector<ScreenUpdateCallBack*> all(updateCallBacks.begin(),updateCallBacks.end());
    currentIteration = &all;
    
    jsrlog("Updater::tick\n");
    vector<ScreenUpdateCallBack*>::iterator it = all.begin();
    while(it != all.end())
    {
	ScreenUpdateCallBack* i = (*it);
	if (i) {
	    i->screenUpdateSoon();
	}
	++it;
    }
    
    currentIteration = NULL;
    Application::current->redraw();
}

ScreenUpdateCallBack::ScreenUpdateCallBack()
{
    registered = false;
}

ScreenUpdateCallBack::~ScreenUpdateCallBack()
{
    unlisten();
}

void ScreenUpdateCallBack::listen()
{
    if (registered) return;
    Application::current->updater.insertCallBack(this);
    registered = true;
}

void ScreenUpdateCallBack::unlisten()
{
    if (!registered) return;
    Application::current->updater.removeCallBack(this);
    registered = false;
}

