#ifndef WIDGET_H
#define WIDGET_H 1

#include "jsreflect.h"

#include "Graphs.h"
#include "ImageFactory.h"

#include <string>
#include <vector>
#include <list>

class Layout;

class InputEventHandler : public JSReflection::Object
{
    JS_EXPORTED_CLASS
    
    Widget * father;
    bool allowUnvisibleFlag;
    std::string name;
    std::string action;
public:
    InputEventHandler(Widget * father);
    virtual ~InputEventHandler();

    Widget * getFather() const;
    
    void setName(const std::string & name);
    std::string getName() const;
    
    void setAction(const std::string & name);
    std::string getAction() const;
    
    void setAllowUnvisible(bool b);
    bool allowUnvisible() const;
    
    bool match(const std::string & event, bool visibility);
    
    bool call(const std::string & event);
};

typedef struct fatherDataStruct {
    int int_data_1;
    int int_data_2;
};

class Widget : public JSReflection::Object, public Notifiable
{
    friend class Painter;
    friend class InputEventHandler;
    friend class Layout;
    friend class GridLayout;
    friend class FullLayout;
    JS_EXPORTED_CLASS
    
    Widget * father;
    
    Widget * modalFor;
    std::list<Widget*> modals;
public:
    fatherDataStruct fatherData;
    
protected:
    virtual void initFatherData(Widget * child);
    virtual void looseFatherData(Widget * child);
   
    Layout * layout;
    
    bool visible;
    
    Color color;
    int x, y, w, h;
    int roundShape;
    int childShiftX,childShiftY;
    
    bool layedOut;
    int layoutId1;
    int layoutId2;
    int backgroundHAlign,backgroundVAlign;
    bool backgroundAsThumb;
    
    std::string text;
    Font * textFont;
    Color textColor;
    int textHAlign,textVAlign;

    std::string style;
    std::string substyle;
    
    std::list<Widget*> childs;

    std::list<InputEventHandler*> inputs;
        
    void addInputEventHandler(InputEventHandler * ieh);
    void removeInputEventHandler(InputEventHandler * ieh);
    
    std::string background_file;
    ImageUser background;
    virtual ~Widget();
    
    virtual void notified(Notifier * v);
    void notifyGlobalVisibleChanged();
    virtual void globalVisibleChanged();
    void backgroundBind();
public:
    Widget();
    virtual void destroy();
    
    virtual void draw(Surface * dest,const ScreenArea * clip);
    
    void getAbsolutePos(int & x,int & y);
    
    void updateLayout();
    
    virtual void removeChilds();
    
    void setFather(Widget * w);
    Widget * getFather() const;
    void setPosition(int x,int y,int w,int h);
    
    void setColor(int r,int g,int b);
    void setColor(const Color & c);
    
    void setAlpha(unsigned int a);
    unsigned int getAlpha() const;
    void setColor(unsigned int c);
    unsigned int getColor() const;

    void setRoundShape(int i);
    int getRoundShape() const;
    
    void setTextColor(unsigned int c);
    unsigned int getTextColor() const;
    
    void setGlobalName(const std::string & text);
    std::string getGlobalName() const;
    
    void setText(const std::string & text);
    const std::string & getText() const;

    void setTextVAlign(int i);
    int getTextVAlign() const;

    void setTextHAlign(int i);
    int getTextHAlign() const;
    
    
    void setTextFont(Font * f);
    Font * getTextFont() const;
    
    const std::string & getStyle() const;
    void setStyle(const std::string & value);

    const std::string & getSubStyle() const;
    
    void subStyleAdd(const std::string & name);
    void subStyleRemove(const std::string & name);
    bool hasSubStyle(const std::string & name) const;
    void setSubStyle(const std::string & name, bool value);
    
    void setVisible(bool b);
    bool getVisible() const;

    void grabInput(Widget * other);
    void ungrabInput();
    
    bool handleInputEvent(const std::string & name,bool visible);
    
    void setBackground(const std::string& s);
    std::string getBackground() const;
    
    void setBackgroundAsThumb(bool b);
    bool getBackgroundAsThumb() const;
    
    void setBackgroundVAlign(int i);
    int getBackgroundVAlign() const;

    void setBackgroundHAlign(int i);
    int getBackgroundHAlign() const;

    int getTop() const;
    int getLeft() const;
    int getWidth() const;
    int getHeight() const;
    void setTop(int i);
    void setLeft(int i);
    void setWidth(int i);
    void setHeight(int i);
    
    void setLayout(Layout * l);
    
    std::string debug(int n) const;
    bool loadXml(const char * xmlFileName);
    
    // Return true if a screen update should be programmed
    bool checkScreenUpdate() const;
    // Return true if the widget is visible.
    bool isGlobalVisible() const;
    // Ask for a screen update
    void askScreenUpdate() const;
};


#endif

