#include <libxml/parser.h>
#include <libxml/parserInternals.h>

#include <stdexcept>
#include <stdexcept>

#include "jsreflect.h"
#include "XmlParser.h"

XmlParsableItem::XmlParsableItem(JSReflection::Context * c)
{
    context = c;
    autokill = true;
    parser = 0;
}

XmlParsableItem::~XmlParsableItem()
{
    if(!autokill) {
	jsrlog("killed while in autokill !\n");
    }
}

void XmlParsableItem::setParser(XmlParser * p)
{
    parser = p;
}

void XmlParsableItem::init()
{
}

void XmlParsableItem::setAutoKill(bool b)
{
    autokill = b;
}

bool XmlParsableItem::getAutoKill() const
{
    return autokill;
}


void XmlParsableItem::addAttribute(const std::string & name,const std::string & value)
{
    // Just ignore...
    fprintf(stderr,"XML : Ignore attribute %s\n",name.c_str());
}

void XmlParsableItem::addText(const std::string & text)
{
    // Just ignore
    for(unsigned int  i = 0 ; i < text.length(); ++i) {
        if ((text[i] != '\t') && (text[i] != '\012') && (text[i] != ' ') && (text[i] != '\015')) { 
            fprintf(stderr,"XML : Ignore text  %s\n",text.c_str());
            return;
        }
    }
}

XmlParsableItem * XmlParsableItem::newChild(const std::string & name)
{
    fprintf(stderr,"XML : Ignore child %s\n",name.c_str());
    return new XmlParsableItem(getContext());
}

JSReflection::Context * XmlParsableItem::getContext() const
{
    return context;
}

void XmlParsableItem::done()
{
}

XmlJsObject::XmlJsObject(JSReflection::Context * c) : XmlParsableItem(c)
{
    object = 0;
}

XmlJsObject::~XmlJsObject()
{
}

void XmlJsObject::setObject(JSReflection::Object * vobject)
{
    object = vobject;
}

void XmlJsObject::addAttribute(const std::string & name,const std::string & value)
{
    if (!object) throw std::logic_error("XmlJsObject has no object");
    if (name == "knownAs") {
	object->evaluate((value + "=this;").c_str(),parser->getLocation());
    }
    if (!object->setProperty(name.c_str(),value.c_str(),parser->getLocation())) {
	fprintf(stderr,"ignored property %s\n",name.c_str());
    }
}

XmlParsableItem * XmlJsObject::newChild(const std::string & name)
{
    if (name == "script") {
	XmlJsScript * xjs = new XmlJsScript(getContext());
	xjs->setObject(object);
	return xjs;
    }
    return XmlParsableItem::newChild(name);
}

XmlJsScript::XmlJsScript(JSReflection::Context * c) : XmlParsableItem(c) ,jssi(JSReflection::JSSourceInfo::unknown)
{
    object = 0;
    scope = 0;
    scopeProtected = false;
}

XmlJsScript::~XmlJsScript()
{
    if (scopeProtected) {
	JS_RemoveRoot(object->GetContext()->GetJSContext(), &scope);
    }
}

void XmlJsScript::init()
{
    if (parser) {
	jssi = parser->getLocation();
    }
}

void XmlJsScript::addAttribute(const std::string & name,const std::string & value)
{
    //printf("set %s to %s\n",name.c_str(),value.c_str());
    if (!object) throw std::logic_error("XmlJsScript has no object");
    if (name == "scope") {
	jsval val;
	if (object->GetContext()->ExecuteScript(value.c_str(),object,&val,jssi)) {
	    if (JSVAL_IS_OBJECT(val)) {
		scope = JSVAL_TO_OBJECT(val);
	    } else {
		jsrlog("invalide scope value (must be an object !)\n");
	    }
	} else {
	    jsrlog("invalide scope\n");
	}
	return;
    }
    return XmlParsableItem::addAttribute(name,value);
}

void XmlJsScript::setObject(JSReflection::Object * vobject)
{
    object = vobject;
    scopeProtected = true;
    JS_AddRoot(object->GetContext()->GetJSContext(), &scope);
    scope = object->GetJSObject();;
}


XmlParsableItem * XmlJsScript::newChild(const std::string & name)
{
    if (name == "script") {
	XmlJsScript * xjs = new XmlJsScript(getContext());
	xjs->setObject(object);
	return xjs;
    }
}

void XmlJsScript::addText(const std::string & text)
{
    if (!object) throw std::logic_error("XmlJsScript has no object");
    script+=text;
}

void XmlJsScript::done()
{
    if (!object) throw std::logic_error("XmlJsScript has no object");
    if (script.length()) {
    	object->GetContext()->ExecuteScript(script.c_str(),scope,0,jssi);
    }
}
