#include "i18n.h"
#include "jsreflect.h"
#include "XmlParser.h"

#include <map>

std::string lang;
std::map<std::string, std::string> dictionary;


/*
<item><c>Hello</c>
			<fr>Bonjour</fr>
</item>
*/
class I18N_XmlItem;

class I18N_Translated_XmlItem :public XmlParsableItem{
    I18N_XmlItem * it;
    std::string lang;
    std::string text;
public:
    I18N_Translated_XmlItem(I18N_XmlItem * i,const std::string name) : XmlParsableItem() {
	it = i;
	lang = name;
    }

    
    virtual void addText(const std::string & t) {
	text += t;
    }
    
    virtual void done();
};

class I18N_XmlItem : public XmlParsableItem {
    friend class I18N_Translated_XmlItem;
    
    std::map<std::string, std::string> translations;
public:
    I18N_XmlItem() : XmlParsableItem()
    { }
    
    virtual ~I18N_XmlItem()
    { }
    
    virtual XmlParsableItem * newChild(const std::string & name) {
	return new I18N_Translated_XmlItem(this,name);
    }
    
    virtual void done() {
	typeof (translations.find("c")) c = translations.find("c");
	if (c != translations.end()) {
	    typeof (translations.find("c")) trans = translations.find(lang);
	    if (trans != translations.end()) {
		dictionary[c->second] = trans->second;
	    } else {
		fprintf(stderr,"missing %s translation for %s\n", lang.c_str(), c->second.c_str());
	    }
	} else {
	    fprintf(stderr,"%s:%d:missing c entry in translation.\n",parser->getLocation().getName().c_str(),parser->getLocation().getLine());
	}
    }
};

void I18N_Translated_XmlItem::done()
{
    it->translations[lang] = text;
}


class I18N_In_XmlItem: public XmlParsableItem {
public:
    I18N_In_XmlItem() : XmlParsableItem()
    { }
    
    virtual XmlParsableItem * newChild(const std::string & name) {
	if (name == "item") {
	    return new I18N_XmlItem();
	} else {
	    return XmlParsableItem::newChild(name);
	}
    }
};

class I18N_Root_XmlItem: public XmlParsableItem {
public:
    I18N_Root_XmlItem() : XmlParsableItem()
    { }
    
    virtual XmlParsableItem * newChild(const std::string & name) {
	if (name == "catalog") {
	    return new I18N_In_XmlItem();
	} else {
	    return XmlParsableItem::newChild(name);
	}
    }
};

void setLang(const std::string & id)
{
    lang = id;
    dictionary.clear();
}

void loadCatalog(const std::string & path)
{
    XmlParser parser;
    parser.pushItem(new I18N_Root_XmlItem());
    parser.parseDocument(path);
}

std::string i18n(const std::string & v,int argc,std::string ** argv)
{
    const std::string * rslt;
    typeof(dictionary.find(v)) it = dictionary.find(v);
    if (it == dictionary.end()) {
	fprintf(stderr,"missing translation for %s\n",v.c_str());
	rslt = &v;
    } else {
	rslt = &(it->second);
    }
    if (argc) {
	std::string vrslt;
	unsigned sze = (*rslt).length();
	unsigned i = 0;
	
	
	while(i < sze) {
	    if (((*rslt)[i] == '$') && (i + 1 <sze) && ((*rslt)[i+1] >= '1') && ((*rslt)[i + 1] <= '9'))  {
		int t = (int) ((*rslt)[i + 1] - '1');
		if (t < argc) {
		    vrslt += *(argv[t]);
		}
		    
		i += 2;
	    } else {
		vrslt += (*rslt)[i];
		i++;
	    }
	}
	
	return vrslt;
    } else {
	return *rslt;
    }
}

static int setI18NLang(const std::string & val)
{
    setLang(val);
    return 0;
}

static int loadI18NCatalog(const std::string & val)
{
    loadCatalog(val);
    return 0;
}

JS_NOCONTEXT_GLOBAL_FUNCTION_1(setI18NLang,int,"%i",0,JSReflection::JS_String,"%s",0)
JS_NOCONTEXT_GLOBAL_FUNCTION_1(loadI18NCatalog,int,"%i",0,JSReflection::JS_String,"%s",0)

