/***************************************************************************
 *   Copyright (C) 2004 by ludovic p                                       *
 *   pludov@nnx.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "jsreflect.h"


using namespace JSReflection;

static int objectid = 0;


Object::Object( Context * c , bool preserved,int obid)
{
    ReflectionContext = c;
    
    
    Prev = c->Last;
    Next = 0;

    if ( c->Last )
    {
        c->Last->Next = this;
    }
    else
    {
        c->First = this;
    }
    c->Last = this;

    Counterpart = 0;
    PreserveCounterpart = preserved;
    
    FirstTargeting = 0;
    LastTargeting = 0;
    FirstSourcing = 0;
    LastSourcing = 0;
    
    RelayEventTo = 0;
    RelayEventId = 0;
    Relayed = 0;
    
    debugFlag = false;
    if (obid == -1) {
	debug = (++objectid);
    } else {
	debug = obid;
    }
    if (debugFlag) {
	jsrlog("[%#08x] : created\n",debug);
    }
}

Object::~Object()
{
    if ((!isDebugZombi()) && debugFlag) {
	jsrlog("[DEBUGGC] Object %#08x JS %#08x deleted\n",this,Counterpart);
    }

    if (debugFlag) {
	jsrlog("[%#08x] : ~deleted\n",debug);
    }
    DettachContext();
    
    while(FirstTargeting) {
	FirstTargeting->TargetLost();
    }
    while(FirstSourcing) {
	FirstSourcing->SourceLost();
    }
    
    while(RelayEventTo) {
	Unlisten(RelayEventTo[0]);
    }
    
    while(Relayed) {
	Relayed[0]->Unlisten(this);
    }
}

bool Object::isDebugZombi() const
{
    return false;
}

bool Object::setProperty(const char * name,jsval v)
{
    if (JS_SetProperty(ReflectionContext->JSGlobalContext,GetJSObject(),name,&v)) {
    	return true;
    }
    return false;
}


bool Object::setProperty(const char * name,Object * obj)
{
    jsval v;
    v = (OBJECT_TO_JSVAL(obj->GetJSObject()));
    if (JS_SetProperty(ReflectionContext->JSGlobalContext,GetJSObject(),name,&v)) {
	return true;
    }
    return false;
}

bool Object::evaluate(const char * value,const JSSourceInfo & fsfi)
{
    jsval returnValue;
    
    if (fsfi.getName() == "unknown") {
	jsrlog("Object::evaluate %s:%d\n",fsfi.getName().c_str(),fsfi.getLine());
    }
    if (JS_EvaluateScript(ReflectionContext->JSGlobalContext,GetJSObject(),value,strlen(value),fsfi.getName().c_str(),fsfi.getLine(),&returnValue)) {
    	return true;
    }
    fprintf(stderr,"Evaluate %s failed\n",value);
    return false;
}

bool Object::setProperty(const char * name,const char * value,const JSSourceInfo & fsfi)
{
    jsval returnValue;

    if (fsfi.getName() == "unknown") {
	jsrlog("Object::setProperty %s:%d\n",fsfi.getName().c_str(),fsfi.getLine());
    }
    
    if (JS_EvaluateScript(ReflectionContext->JSGlobalContext,GetJSObject(),value,strlen(value),fsfi.getName().c_str(),fsfi.getLine(),&returnValue)) {
	if (JS_SetProperty(ReflectionContext->JSGlobalContext,GetJSObject(),name,&returnValue)) {
	    return true;
	}
    }
    fprintf(stderr,"Setting %s to %s failed\n",name,value);
    return false;
}

void Object::AddRelay(Object * proxyTo,const char * id)
{
    int RelayCount = 0;
    
    if (RelayEventTo) {
	while(RelayEventTo[RelayCount]) {
	    RelayCount++;
	}
    }
    
    RelayEventTo = (Object**) realloc(RelayEventTo,sizeof(Object*) * (RelayCount+2));
    RelayEventId = (const char **) realloc(RelayEventId,sizeof(const char **) * (RelayCount+1));
    RelayEventTo[RelayCount] = proxyTo;
    RelayEventTo[RelayCount+1] = 0;
    RelayEventId[RelayCount] = id;
}

void Object::DelRelay(Object * proxyTo)
{
    int i = 0;

    if (!RelayEventTo) return;
    while(RelayEventTo[i]) {
	if (RelayEventTo[i] == proxyTo) {
	    int j = i + 1;
	    while(RelayEventTo[j]) {
		j++;
	    }
	    RelayEventTo[i] = RelayEventTo[j-1];
	    if (RelayEventId[i] && (RelayEventId[i][0] == '#')) {
		free((void*)RelayEventId[i]);
	    }
	    RelayEventId[i] = RelayEventId[j-1];
	    // We currently have j items in RelayEventTo
	    if (i == 0 && j == 1) {
		free(RelayEventTo);
		free(RelayEventId);
		RelayEventTo = 0;
		RelayEventId = 0;
	    } else {
		RelayEventTo = (Object**)realloc((void*)RelayEventTo,sizeof(Object*)*(j-1+1));
		RelayEventId = (const char **)realloc((void*)RelayEventId,sizeof(const char *)*(j-1));
	    }
	    return;
	}
	i++;
    }
}

void Object::AddRelayed(Object * o)
{
    int RelayedCount = 0;
    if (Relayed) {
	while(Relayed[RelayedCount]) {
	    RelayedCount++;
	}
    }
    Relayed = (Object**) realloc(Relayed,sizeof(Object*)*(RelayedCount+2));
    Relayed[RelayedCount] = o;
    Relayed[RelayedCount + 1] = 0;
}

void Object::DelRelayed(Object * proxyTo)
{
    int i = 0;

    if (!Relayed) return;
    while(Relayed[i]) {
	if (Relayed[i] == proxyTo) {
	    int j = i + 1;
	    while(Relayed[j]) {
		j++;
	    }
	    
	    Relayed[i] = Relayed[j-1];
	    
	    // We currently have j items in RelayEventTo
	    if (i == 0 && j == 1) {
		free(Relayed);
		Relayed = 0;
		
	    } else {
		Relayed = (Object**)realloc((void*)Relayed,sizeof(Object*)*(j-1+1));
	    }
	    
	    return;
	}
	i++;
    }
}

void Object::Listen(const char * id,Object * proxyTo)
{
    AddRelay(proxyTo,id);
    proxyTo->AddRelayed(this);
}

void Object::Listen(const char * id,Object * t,Closure * cl)
{
    char * realId;
    
    realId = (char*)malloc(strlen(id)+2);
    strcpy(realId,"#");
    strcat(realId,id);
    
    Listen(realId,cl);
    cl->Listen(0,t);
}

void Object::Unlisten(Object * proxyTo)
{
    DelRelay(proxyTo);
    proxyTo->DelRelayed(this);
    
    RelayedLost(proxyTo);
    proxyTo->RelayerLost(this);
}

void Object::RelayedLost(Object * o)
{
}

void Object::RelayerLost(Object * o)
{
}

JSObject * Object::GetJSObject( bool autocreate )
{
    if (isDebugZombi()) {
	jsrlog("[DEBUGGC] %#x %#x : Zombi::GetJSObject called!\n",Counterpart,this);
    }
    if ( Counterpart )
    {
        return Counterpart;
    }


    JSObject * instance = ReflectionContext->NewInstance( GetClassId() );

    if (debugFlag) {
	jsrlog("[%#08x] (%s) : associated to %#x\n",debug,GetClassName(),instance);
    }
    
    JS_SetPrivate( ReflectionContext->JSGlobalContext, instance, ( JSObject* ) this );

    SetCounterpart( instance );
    if ((!isDebugZombi()) && debugFlag) {
	jsrlog("[DEBUGGC] Object %#08x JS %#08x associated\n",this,Counterpart);
    }
    return Counterpart;
}

const char * Object::GetClassName() const
{
    return GetClassId()->GetName();
}

void Object::SetCounterpart( JSObject * o )
{
    if ( Counterpart == o )
    {
        return ;
    }
    if ( PreserveCounterpart && ReflectionContext )
    {
        if ( o )
        {
            JS_SetPrivate( ReflectionContext->JSGlobalContext, o, ( void* ) this );
            if ( Counterpart )
            {
            	// Remove private for old
                JS_SetPrivate( ReflectionContext->JSGlobalContext, Counterpart, 0 );
                Counterpart = o;
            }
            else
            {
                Counterpart = o;
                // Was not a js root (since counterpart was null)
		char buffer[512];
		snprintf(buffer,512,"[%#x](%s)",this,GetClassName());
		JS_AddNamedRoot( ReflectionContext->JSGlobalContext, &Counterpart, strdup(buffer));
            }
        }
        else
        {
            if ( Counterpart )
            {
		// Counterpart becomes a zombi !
		ReflectionContext->newZombi(Counterpart,this);
		
                JS_RemoveRoot( ReflectionContext->JSGlobalContext, &Counterpart );
            }
            Counterpart = o;
        }
    }
    else
    {
    	// Same but without root stuff
        if ( Counterpart )
        {
	    // Counterpart becomes a zombi !
	    ReflectionContext->newZombi(Counterpart,this);
            // JS_SetPrivate( ReflectionContext->JSGlobalContext, Counterpart, 0 );
        }
        Counterpart = o;
        if ( Counterpart )
        {
            JS_SetPrivate( ReflectionContext->JSGlobalContext, Counterpart, ( void* ) this );
        }
    }
}

void Object::DettachContext()
{
    SetCounterpart( 0 );
    if ( !ReflectionContext )
    {
        return ;
    }
    if ( Prev )
    {
        Prev->Next = Next;
    }
    else
    {
        ReflectionContext->First = Next;
    }

    if ( Next )
    {
        Next->Prev = Prev;
    }
    else
    {
        ReflectionContext->Last = Prev;
    }

    ReflectionContext = 0;
}

Context * Object::GetContext() const
{
    return ReflectionContext;
}

bool Object::debugGC() const {
    return debugFlag;
}

void Object::setDebugGC(bool b) {
    if (debugFlag != b) {
	debugFlag = b;
	if (debugFlag) {
	    jsrlog("[DEBUGGC] Object %#08x JS %#08x debug on\n",this,Counterpart);
	} else {
	    jsrlog("[DEBUGGC] Object %#08x JS %#08x debug off\n",this,Counterpart);
	}
    }
}

bool Object::getDebugGC() const {
    return debugFlag;
}

bool Object::call(const char * name,void * retvalue, const char * retformat, JSReflection::ClassInfo * retspecifier,
	    		const char * formats, ...)
{
    int i,cpt;
    i = 0; cpt = 0;
    while(formats[i]) {
	if (formats[i] == '%') {
	    cpt++;
	}
	i++;
    }
    
    jsval args[cpt];
    
    if (cpt) {
	std::string argformat[cpt];
	
	int lastpos;
	int curpara;
	
	lastpos = 0;
	curpara = 0;
	i = 0;
	do {
	    if ((formats[i] == '%') || (!formats[i])) {
		if (lastpos != i) {
		    argformat[curpara] = std::string(formats+lastpos, i - lastpos);
		    lastpos = i;
		    curpara ++;
		}
	    }
	    i++;
	} while(formats[i-1]);
	
	
	va_list arglist;
	va_start(arglist,formats);
	
	for(i = 0; i < cpt; ++i) {
	    void * input = va_arg(arglist,void*);
	    JSReflection::ClassInfo * specifier = va_arg(arglist,JSReflection::ClassInfo*);
	    if (!GetContext()->ValueToJS(input, args+i,argformat[i].c_str(),specifier)) {
		fprintf(stderr,"call %s : convertion %s failed\n",name,argformat[i].c_str());
		return false;
	    }
	}
    }
    jsval rval;
    if (!JS_CallFunctionName(GetContext()->GetJSContext(),GetJSObject(),name,cpt,args,&rval)) {
	fprintf(stderr,"call %s : failed\n",name);
	return false;
    }
    if (retvalue) {
	if (!GetContext()->ValueFromJS(&rval,retvalue,retformat,retspecifier)) {
	    fprintf(stderr,"call : %s : failed return convertion\n",name);
	    return false;
	}
    }
    return true;
}

void Object::HandleEvent(Event * a)
{
    if (RelayEventTo && (a->GetTarget() == this)) {
	int i = 0;
	while(RelayEventTo[i]) {
	    const char * id;
	    if (id = RelayEventId[i]) {
		if (id[0] == '#') id++;
		if (a->IsA(id)) {
		    RelayEventTo[i]->HandleEvent(a);
		}
	    }
	    i++;
	}
    }
}



//template JSReflection::SpecificClassInfo<Object>;
class Object_ClassInfo:public JSReflection::SpecificClassInfo<Object> {
public:
    Object_ClassInfo(): JSReflection::SpecificClassInfo<Object>( "Object", 0) {}
};
JSReflection::ClassInfo * Object::ClassInfo = new Object_ClassInfo ();
const JSReflection::ClassInfo * Object::GetClassId() const { return Object::ClassInfo; }

JS_NOCONTEXT_OBJECT_VOID_FUNCTION_3(Object,Listen,
				JS_String,"%s",0,
				Object *,"%x",Object::ClassInfo,
				Closure*,"%x",Closure::ClassInfo);
JS_NOCONTEXT_OBJECT_FUNCTION_0(Object,GetClassName,JS_String,"%s",0)
//JS_NOCONTEXT_OBJECT_VOID_FUNCTION_0(Object,dumpRoots)
JS_RW_PROPERTY(Object,debugGC,getDebugGC,setDebugGC,bool,"%b",0);


